/***************************************************************************
*                 _                                _                       *
*                /_\       Chess Board Engine     /_\                      *
*                \_/      -------------------     \_/                      *
*                /_\        Based on Rules        /_\                      *
*                |_|      -------------------     |_|                      *
*               /___\      GNU GPL Version 2     /___\                     *
*                                                                          *
***************************************************************************/
/* Header Files */
#include "engine.h"

/* Constructor */
Game::Game() {}

/* Window */
static WINDOW *winF;
static WINDOW *winE;
unsigned int usecs = 200000;

//-----------------------------------------------------------------------------

/* Material */
void Game::material()
{	
	/* White */
	cengine[5][1][1][0] = '0';
	cengine[5][1][8][0] = '+';
	
	cengine[8][1][5][0] = 'R';
	cengine[7][1][3][0] = 'N';
	cengine[6][1][4][0] = 'B';
	cengine[5][1][7][0] = 'K';
	cengine[4][1][6][0] = 'Q';
	cengine[3][1][4][0] = 'B';
	cengine[2][1][3][0] = 'N';
	cengine[1][1][5][0] = 'R'; 
			
	cengine[8][2][2][0] = 'P'; 
	cengine[7][2][2][0] = 'P';
	cengine[6][2][2][0] = 'P';
	cengine[5][2][2][0] = 'P';
	cengine[4][2][2][0] = 'P';
	cengine[3][2][2][0] = 'P';
	cengine[2][2][2][0] = 'P';
	cengine[1][2][2][0] = 'P'; 
				
	/* Black */
	cengine[5][8][1][1] = '0';
	cengine[5][8][8][1] = '+';
	
	cengine[8][8][5][1] = 'R';
	cengine[7][8][3][1] = 'N';
	cengine[6][8][4][1] = 'B';
	cengine[5][8][7][1] = 'K';
	cengine[4][8][6][1] = 'Q';
	cengine[3][8][4][1] = 'B';
	cengine[2][8][3][1] = 'N';
	cengine[1][8][5][1] = 'R'; 
			
	cengine[8][7][2][1] = 'P'; 
	cengine[7][7][2][1] = 'P';
	cengine[6][7][2][1] = 'P';
	cengine[5][7][2][1] = 'P';
	cengine[4][7][2][1] = 'P';
	cengine[3][7][2][1] = 'P';
	cengine[2][7][2][1] = 'P';
	cengine[1][7][2][1] = 'P'; 
}

//-----------------------------------------------------------------------------

/* Notation */
void Game::notation()
{	
	/* Calculation */
	X = 0;Y = 10;

	/* a - h */
	emove[1+X] = 'a';
	emove[2+X] = 'b';
	emove[3+X] = 'c';
	emove[4+X] = 'd';
	emove[5+X] = 'e';
	emove[6+X] = 'f';
	emove[7+X] = 'g';
	emove[8+X] = 'h';
	
	/* 1 - 8 */
	emove[1+Y] = '1';
	emove[2+Y] = '2';
	emove[3+Y] = '3';
	emove[4+Y] = '4';
	emove[5+Y] = '5';
	emove[6+Y] = '6';
	emove[7+Y] = '7';
	emove[8+Y] = '8';
}

//-----------------------------------------------------------------------------

/* Engine */
void Game::engine(int T1,int T2,int L1)
{	
	/* Move From */
	amove[0] = emove[F1+X];
	amove[1] = emove[F2+Y];
	
	/* Calculation */
	S1 = F1 + T1;
	S2 = F2 + T2;
	
	/* 64 Felder */
	if(((S1 >= 1) && ( S1 <= 8)) && ((S2 >= 1) && (S2 <= 8)))
	{			
		/* Move to */		
		amove[2]  = emove[X+S1];
		amove[3]  = emove[Y+S2];
							
		mvwprintw(winE,L1,13,"[%c%c/%c%c]",amove[0],amove[1],amove[2],amove[3]);
	
		/* Wait Time */
		usleep(usecs);
		
		/* Windows Refresh */
		wrefresh(winF);
		wrefresh(winE);
	}
}

//-----------------------------------------------------------------------------

/* King */
void Game::king(int T1,int T2)
{
	if(((T1 >= -1) && (T1 <= 1)) && ((T2 >= -1) && (T2 <= 1)) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2,L1);
	}
}

//-----------------------------------------------------------------------------

/* Queen */
void Game::queen(int T1,int T2)
{
	if((((T1 == 0) || (T2 == 0)) && ((T1 != 0) || (T2 != 0))))
	{
		engine(T1,T2,L1);
	}
	if((((T1 == T2) || ((T1 + T2) == 0))) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2,L1);
	}
}

//-----------------------------------------------------------------------------

/* Turm */
void Game::turm(int T1,int T2)
{
	if((((T1 == 0) || (T2 == 0)) && ((T1 != 0) || (T2 != 0))))
	{
		engine(T1,T2,L1);
	}
}

//-----------------------------------------------------------------------------

/* Bishop */
void Game::bishop(int T1,int T2)
{
	if((((T1 == T2) || ((T1 + T2) == 0))) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2,L1);
	}
}

//-----------------------------------------------------------------------------

/* Springer */
void Game::springer(int T1,int T2)
{
	if((((T2 == -2) || (T2 == +2)) && ((T1 == -1) || (T1 == +1))))
	{
		engine(T1,T2,L1);
	}
	if((((T1 == -2) || (T1 == +2)) && ((T2 == -1) || (T2 == +1))))
	{
		engine(T1,T2,L1);
	}
}

//-----------------------------------------------------------------------------

/* People B */
void Game::people1(int T1,int T2)
{
	if(((T1 == 0)) && (T2 == -2) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2,L1);
	}
	if(((T1 >= -1) && (T1 <= +1)) && (T2 == -1) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2,L1);
	}
}

//-----------------------------------------------------------------------------

/* People W */
void Game::people2(int T1,int T2)
{	
	if(((T1 == 0)) && (T2 == +2) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2,L1);
	}
	if(((T1 >= -1) && (T1 <= +1)) && (T2 == +1) && ((T1 != 0) || (T2 != 0)))
	{
		engine(T1,T2,L1);
	}
}

//-----------------------------------------------------------------------------

/* Sort */
void Game::sort(char *nmove)
{
	/* Window Box */
	winF = newwin(11,76,13,2);
	winE = newwin(9,74,14,3);
	box(winF,ACS_VLINE,ACS_HLINE);
	scrollok(winE,true);
	
	/* Windows Refresh */
	touchwin(stdscr);
	refresh();
	wrefresh(winF);
	wrefresh(winE);

	/* Variables */
	FIGU = 1;
	L1 = -1;
	
	/* Function */
	material();
	notation();
	
	/* Matrix */
	matrix[1][0][0][0] = '0';
	matrix[8][0][0][1] = '+';
	matrix[7][1][0][0] = 'K';
	matrix[6][1][0][1] = 'Q';
	matrix[5][0][1][0] = 'R'; 
	matrix[4][1][1][0] = 'B';
	matrix[3][0][1][1] = 'N';
	matrix[2][1][1][1] = 'P'; 
	
	/* B / W */
	for(E1 = 0, C1 = 1; E1 <= 1; E1++)
	{
	/* Notation */
	for(E2 = 0, C2 = 2; E2 <= 1; E2++)
	{
	L1++; L2 = 15;
	/* Direction */
	for(E3 = 0, C3 = 8; E3 <= 1; E3++)
	{
	/* Material */
	for(E4 = 0, C4 = 4; E4 <= 1; E4++)
	{
	/* Black and White */
	for(E5 = 0, C5 = 0; E5 <= 1; E5++)
	{
	/* Logic */		
	if(((E1 || E2) || (E3 || E4)) || E5)
	{
	for(FIGU = -2, FELD = 0; FIGU < 0; FIGU += 2)
	{
		/* Engine Deep */
		FELD = FELD | FIGU;
		
		/* Counter */
		if(L1 >= 8)
		{
			L1 = 8;
		}
		
		/* 1. Engine Info */
		mvwprintw(winE,L1,0,"%+3d %d%d%d%d%d ", 
		FELD, E1+3, E2+5, E3, E4+1, E5);
		
		/* Material */
		for(F1 = 1;F1 <= 8;F1++)
		{	
		for(F2 = 1;F2 <= 8;F2++)
		{				
			/* Direction */
			for(T1 = -7;T1 <= 7;T1++)
			{
			for(T2 = -7;T2 <= 7;T2++)
			{
				/* Chess Diplomatic */
				if(cengine[F1][F2][8][E5] && matrix[8][E2][E3][E4])
				{	
					mvwprintw(winE,L1,11,"%c",cengine[F1][F2][8][E5]);
					king(T1,T2);
				}
				/* Rochade */
				if(cengine[F1][F2][1][E5] && matrix[1][E2][E3][E4])
				{	
					mvwprintw(winE,L1,11,"%c",cengine[F1][F2][1][E5]);
					king(T1,T2);
				}
				/* K:  T0 | T0  */
				if(cengine[F1][F2][7][E5] && matrix[7][E2][E3][E4])
				{	
					mvwprintw(winE,L1,11,"%c",cengine[F1][F2][7][E5]);
					king(T1,T2);
				}
				/* Q:  T2 | T0  */
				if(cengine[F1][F2][6][E5] && matrix[6][E2][E3][E4])
				{
					mvwprintw(winE,L1,11,"%c",cengine[F1][F2][6][E5]);
					queen(T1,T2);
				}	
				/* R:  T2 | T0   */
				if(cengine[F1][F2][5][E5] && matrix[5][E2][E3][E4])
				{
					mvwprintw(winE,L1,11,"%c",cengine[F1][F2][5][E5]);
					turm(T1,T2);	
				}
				/* B:  T2 | T2   */	
				if(cengine[F1][F2][4][E5] && matrix[4][E2][E3][E4])
				{
					mvwprintw(winE,L1,11,"%c",cengine[F1][F2][4][E5]);
					bishop(T1,T2);
				}
				/* N:  T2 | T1   */
				if(cengine[F1][F2][3][E5] && matrix[3][E2][E3][E4])
				{	
					mvwprintw(winE,L1,11,"%c",cengine[F1][F2][3][E5]);
					springer(T1,T2);
				}
				/* P:  T1 | T0   */
				if(cengine[F1][F2][2][E5] && matrix[2][E2][E3][E4])
				{		
					mvwprintw(winE,L1,11,"%c",cengine[F1][F2][2][E5]);
					if(E5 == 0)
					{
						people2(T1,T2);
					}
					if(E5 == 1)
					{
						people1(T1,T2);
					}
				}
		}}}}

		L2 += 6;
		mvwprintw(winE,L1,L2," %c%c%c%c ",amove[0],amove[1],amove[2],amove[3]);

		amove[100] = 0;
		emove[100] = 0;

		/* Engine to Board */
		nmove[0] = amove[0];
		nmove[1] = amove[1];
		nmove[2] = amove[2];
		nmove[3] = amove[3];

		amove[0] = '0';
		amove[1] = '1';
		
		/* Windows Refresh */
		wrefresh(winF);
		wrefresh(winE);
		
	}}}}}}}
}

//-----------------------------------------------------------------------------
